@@ -948,6 +948,24 @@ static PyObject *PyAsap_PotentialGetNbList(PyAsap_PotentialObject *self,
   return nbl;
 }
 
+static PyObject *PyAsap_PotentialCheckUpdateNbList(PyAsap_PotentialObject *self,
+						   PyObject *args, PyObject *kwargs)
+{
+  static char* argnames[] = {"atoms", NULL};
+  PyObject *atoms = NULL;
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O:check_update_neighborlist",
+				   argnames, &atoms))
+    return NULL;
+  CHECK_POT_INIT;
+  bool updated = false;
+  try {
+    updated = self->cobj->CheckAndUpdateNbList(atoms);
+  }
+  POTCATCHEXCEPTION;
+  return PyBool_FromLong((long) updated);
+}
+
+
 static PyObject *PyAsap_PotentialGetName(PyAsap_PotentialObject *self,
 					   PyObject *noargs)
 {
@@ -1032,8 +1050,10 @@ static PyMethodDef PyAsap_PotentialMethods[] = {
    METH_NOARGS, "Get volumes of the atoms (or None if unknown) for calculation of stress."},
   {"calculation_required", (PyCFunction)PyAsap_CalcReq,
    METH_VARARGS, "Check if a calculation is required."},
-  {"get_neighborlist", (PyCFunction)PyAsap_PotentialGetNbList,
+  {"_get_neighborlist", (PyCFunction)PyAsap_PotentialGetNbList,
    METH_NOARGS,  "Return the neighbor list (if any)."},
+  {"check_update_neighborlist", (PyCFunction)PyAsap_PotentialCheckUpdateNbList,
+   METH_VARARGS|METH_KEYWORDS, "Check and possibly update the neighbor list."},
   {"supports_parallel", (PyCFunction)PyAsap_Parallel,
    METH_NOARGS,  "Return True if the calculator supports parallel simulations."},
   {"get_cutoff", (PyCFunction)PyAsap_PotentialGetCutoff,
