@@ -34,6 +34,14 @@ using std::endl;
 using std::flush;
 
 static char mycobject[] = "_asap_parallelatoms_cobject";
+static char invalidghosts[] = "_asap_invalidghosts";
+
+// Exception thrown if ghosts are invalid.  Should always be caught internally.
+class AsapGhostInvalid : public AsapError
+{
+public:
+  AsapGhostInvalid(const char *m) : AsapError(m) {};
+};
 
 // #define REGARRAYINFO   // Print extra debugging info about reg. arrays.
 
@@ -48,6 +56,7 @@ ParallelAtoms::ParallelAtoms(PyObject *py_atoms, int verbose)
 {
   CONSTRUCTOR;
   this->verbose = verbose;
+  hasGhosts = true;
   try {
     DEBUGPRINT;
     migrationCounter = 0;
@@ -159,7 +168,16 @@ void ParallelAtoms::Begin(PyObject *pyatoms, bool allow_reopen, bool postmigrate
       count_inverse_cell = globaldata[4];
       if (decorated && ghost_count != count_positions)
 	{
-	  UpdateGhostData();
+	  try {
+	    UpdateGhostData();
+	  } catch(AsapGhostInvalid &e) {
+	    // The ghosts have explicitly been marked invalid.
+	    double range = get_ghost_range();
+	    DecorateWithGhosts(range);
+	    UpdateGhostData();
+	    // This may also have invalidated nTotalAtoms
+	    nTotalAtoms = mpi->Add(nAtoms);
+	  }
 	  ghost_count = count_positions;
 	}
     }
@@ -249,7 +267,7 @@ void ParallelAtoms::Migrate(bool distributing /* = false */)
   DEBUGPRINT;
 
   // Extract data from Python atoms
-  vector<const char *> array_names; // Sorted list of array names.
+  vector<std::string> array_names; // Sorted list of array names.
   vector<PyArrayObject *> arrays;  // Owned refs to arrays to migrate.
   get_array_names(py_arrays, array_names);
   get_arrays(py_arrays, array_names, arrays);
@@ -567,6 +585,11 @@ static void copynum(vector<asap_z_int> &num,
 
 void ParallelAtoms::UpdateGhostData()
 {
+  // Potentially the ghosts could have been invalidated.  In that case
+  // throw a specific error that can be caught further up.
+  if (PyObject_HasAttrString(py_atoms, invalidghosts))
+    throw AsapGhostInvalid("Ghosts invalid");
+      
   DEBUGPRINT;
   if (verbose)
     cerr << " UG";
@@ -580,13 +603,8 @@ void ParallelAtoms::UpdateGhostData()
   if (mypointer == NULL)
     throw AsapError("ParallelAtoms::UpdateGhostData: failed to get ") <<
       mycobject;
-#if PY_VERSION_HEX < 0x02070000
-  bool OK = PyCObject_Check(mypointer) && (PyCObject_AsVoidPtr(mypointer)
-					   == this);
-#else
   bool OK = (PyCapsule_CheckExact(mypointer) &&
 	     (PyCapsule_GetPointer(mypointer, "asap3.parallelatoms") == this));
-#endif
   Py_DECREF(mypointer);
   if (!OK)
     throw AsapError("ParallelAtoms::UpdateGhostData: Multiple objects are creating ghost atoms.");
@@ -603,7 +621,7 @@ void ParallelAtoms::UpdateGhostData()
   if (py_arrays == NULL)
     throw AsapError("ParallelAtoms::UpdateGhostData: No arrays found.");
   
-  vector<const char *> array_names;          // Sorted list of array names.
+  vector<std::string> array_names;          // Sorted list of array names.
   vector<PyArrayObject *> real_arrays;            // Data on real atoms.
   vector<PyArrayObject *> ghost_arrays;           // Ghost arrays to fill.
   get_array_names(py_ghosts, array_names);
@@ -620,12 +638,12 @@ void ParallelAtoms::UpdateGhostData()
       ASSERT(PyArray_DIM(real_arrays[i], 0) == nAtoms);
       ASSERT(PyArray_DIM(ghost_arrays[i], 0) == nGhosts);
       nBytes += n;
-      if (strcmp(array_names[i], "positions") == 0)
+      if (array_names[i].compare("positions") == 0)
 	{
 	  pos_index = i;
 	  ASSERT(n == sizeof(Vec));
 	}
-      if (strcmp(array_names[i], "numbers") == 0)
+      if (array_names[i].compare("numbers") == 0)
 	  num_index = i;
     }
   ASSERT(pos_index >= 0);  // But no check for num_index: not compulsory.
@@ -795,6 +813,10 @@ void ParallelAtoms::DecorateWithGhosts(double range)
   if (verbose)
     cerr << " DG";
   USETIMER("ParallelPotential::DecorateWithGhosts");
+  // If the ghosts are marked invalid, unmark them
+  if (PyObject_HasAttrString(py_atoms, invalidghosts))
+    PyObject_DelAttrString(py_atoms, invalidghosts);
+  
   // Delete old Ghosts:
   ghosts.resize(nProcessors);
   for (int p = 0; p < nProcessors; p++)
@@ -865,7 +887,7 @@ void ParallelAtoms::extract_ncells(PyObject *pyatoms)
 ////////////////////////////////
 								 
 // Get a sorted list of array names.
-void ParallelAtoms::get_array_names(PyObject *dict, vector<const char *> &names)
+void ParallelAtoms::get_array_names(PyObject *dict, vector<std::string> &names)
 {
   DEBUGPRINT;
   ASSERT(dict != NULL && PyDict_Check(dict));
@@ -878,11 +900,17 @@ void ParallelAtoms::get_array_names(PyObject *dict, vector<const char *> &names)
   for (Py_ssize_t i = 0; i < n; i++)
     {
       PyObject *pyname = PyList_GET_ITEM(keys, i);
-      const char *name = PyUnicode_AsUTF8(pyname);
-      if (name == NULL)
-	throw AsapError("Invalid key is ParallelAtoms' arrays/ghosts.  Not a string?");
-      names[(int)i] = name;
+      if (!PyUnicode_Check(pyname))
+	throw AsapError("Non-string key in ghost atom dictionary.");
+      // Make sure the Unicode object is ready and 8-bit data
+      if ((PyUnicode_READY(pyname) == -1)
+	  || (PyUnicode_KIND(pyname) != PyUnicode_1BYTE_KIND))
+	throw AsapError("Non-ascii key in ghost atom dictionary.");
+      // Now convert to a C++ string
+      names[(int)i].assign((char *)PyUnicode_1BYTE_DATA(pyname),
+			   (size_t)PyUnicode_GET_LENGTH(pyname));
     }
+  Py_DECREF(keys);
   DEBUGPRINT;
 }
 
@@ -894,7 +922,7 @@ void ParallelAtoms::get_array_names(PyObject *dict, vector<const char *> &names)
 ////////////////////////////////
 
 // Get a list of array objects.
-void ParallelAtoms::get_arrays(PyObject *dict, vector<const char *> &names,
+void ParallelAtoms::get_arrays(PyObject *dict, vector<std::string> &names,
 			       vector<PyArrayObject *> &arrays)
 {
   ASSERT(dict != NULL && PyDict_Check(dict));
@@ -902,7 +930,7 @@ void ParallelAtoms::get_arrays(PyObject *dict, vector<const char *> &names,
   arrays.resize(n);
   for (int i = 0; i < n; i++)
     {
-      arrays[i] = AsPyArray(PyDict_GetItemString(dict, names[i]));
+      arrays[i] = AsPyArray(PyDict_GetItemString(dict, names[i].c_str()));
       if (arrays[i] == NULL || !PyArray_Check(arrays[i]))
 	throw AsapError("Invalid data in ParallelAtoms' arrays/ghosts[") <<
 	  names[i] << "].";
@@ -956,16 +984,16 @@ void ParallelAtoms::make_new_arrays(vector<PyArrayObject *> &newarrays,
 
 // Release references to arrays
 void ParallelAtoms::release_arrays(vector<PyArrayObject *> &arrays,
-				    vector<const char *> &names,
+				   vector<std::string> &names,
 				   int maxcount)
 {
   DEBUGPRINT;
-  vector<const char*>::const_iterator name = names.begin();
+  vector<std::string>::const_iterator name = names.begin();
   for (vector<PyArrayObject *>::iterator i = arrays.begin();
        i != arrays.end(); ++i, ++name)
     {
       if (Py_REFCNT(*i) > maxcount)
-	cerr << "ASAP warning: Extra reference detected for " << *name << endl;
+	cerr << "ASAP warning: Extra reference detected for " << name->c_str() << endl;
       CHECKREF(*i);
       Py_DECREF(*i);
     }
@@ -981,7 +1009,7 @@ void ParallelAtoms::release_arrays(vector<PyArrayObject *> &arrays,
 ////////////////////////////////
 
 // Store arrays into a dictionary.
-void ParallelAtoms::store_arrays(PyObject *dict, vector<const char *> &names,
+void ParallelAtoms::store_arrays(PyObject *dict, vector<std::string> &names,
 				 vector<PyArrayObject *> &arrays)
 {
   DEBUGPRINT;
@@ -989,7 +1017,7 @@ void ParallelAtoms::store_arrays(PyObject *dict, vector<const char *> &names,
   int n = names.size();
   for (int i = 0; i < n; i++)
     {
-      int x = PyDict_SetItemString(dict, names[i], (PyObject *) arrays[i]);
+      int x = PyDict_SetItemString(dict, names[i].c_str(), (PyObject *) arrays[i]);
       if (x != 0)
 	throw AsapPythonError();
     }
@@ -1052,15 +1080,9 @@ void ParallelAtoms::set_number_of_ghosts(int nGhosts)
   // First, place a pointer to this object in the Python object to
   // detect if two different ParallelAtoms objects try to use ghosts
   // at the same time.
-#if PY_VERSION_HEX < 0x02070000
-  PyObject *mypointer = PyCObject_FromVoidPtr(this, NULL);
-  if (mypointer == NULL)
-    throw AsapError("Creating PyCObject failed.");
-#else
   PyObject *mypointer = PyCapsule_New(this, "asap3.parallelatoms", NULL);
   if (mypointer == NULL)
     throw AsapError("Creating PyCapsule failed.");
-#endif
   if (PyObject_SetAttrString(py_atoms, mycobject, mypointer) == -1)
     throw AsapError("Failed to set attribute ") << mycobject;
   Py_DECREF(mypointer);
@@ -1070,7 +1092,7 @@ void ParallelAtoms::set_number_of_ghosts(int nGhosts)
   if (py_ghosts == NULL)
     throw AsapError("ParallelAtoms::set_number_of_ghosts:: No ghosts found.");
 
-  vector<const char *> array_names;          // Sorted list of array names.
+  vector<std::string> array_names;          // Sorted list of array names.
   vector<PyArrayObject *> arrays;                 // Ghost arrays to resize.
   get_array_names(py_ghosts, array_names);
   get_arrays(py_ghosts, array_names, arrays);
@@ -1176,35 +1198,6 @@ void ParallelAtoms::CollectFromGhosts(vector<SymTensor> &data)
   ASSERT(ghostAddress - &data[0] == nAtoms + nGhosts);
 }
 
-void ParallelAtoms::SetPrimaryNbLocator(PyObject *nblocator)
-{
-  PyObject *wref = PyWeakref_NewRef(nblocator, NULL);
-  if (wref == NULL)
-    throw AsapPythonError();
-  int x = PyObject_SetAttrString(py_atoms, "_asap_primary_nblist", wref);
-  Py_DECREF(wref);
-  if (x == -1)
-    throw AsapPythonError();
-}
-
-PyObject *ParallelAtoms::GetPrimaryNbLocator(PyObject *pyatoms)
-{
-  if (!PyObject_HasAttrString(pyatoms, "_asap_primary_nblist"))
-    return NULL;   // Not set.  This is not an error!
-  PyObject *wref = PyObject_GetAttrString(pyatoms, "_asap_primary_nblist");
-  if (wref == NULL)
-    throw AsapPythonError();
-  PyObject *locator = PyWeakref_GetObject(wref);
-  Py_DECREF(wref);
-  if (locator == NULL)
-    throw AsapPythonError();
-  if (locator == Py_None)
-    return NULL;  // Weak reference is dead.  This is not an error.
-  Py_INCREF(locator);  // Return a new reference
-  return locator;
-}
-
-
   // Issue a warning if there are no atoms in the simulation, but only the first time.
 void ParallelAtoms::NoAtomsErrorOrWarning()
 {
