@@ -29,6 +29,7 @@
 #include <structmember.h>
 #include <mpi.h>
 #include "mpimodule.h"
+#include "Templates.h"
 
 #ifdef ASAPDEBUG
 #include <stdio.h>
@@ -960,6 +961,7 @@ int PyAsap_InitMpiInterface(PyObject *module)
   MPIType.tp_init = (initproc)InitMPIObject;
   MPIType.tp_new = NewMPIObject;
   MPIType.tp_finalize = (destructor)DelMPIObject;
+  MPIType.tp_finalize = PyAsap_Dealloc;
   if (PyType_Ready(&MPIType) < 0)
     return -1;
 
@@ -968,6 +970,7 @@ int PyAsap_InitMpiInterface(PyObject *module)
   mpi_request_type.tp_methods = mpi_request_methods;
   mpi_request_type.tp_members = mpi_request_members;
   mpi_request_type.tp_finalize = (destructor)mpi_request_finalize;
+  mpi_request_type.tp_finalize = PyAsap_Dealloc;
   if (PyType_Ready(&mpi_request_type) < 0)
     return -1;
 
