@@ -132,6 +132,10 @@ class AsapPotential:
 
     def get_name(self):
         return "asap3." + self._get_name()
+
+    def get_neighborlist(self):
+        """Return the neighbor locator"""
+        return self._get_neighborlist()
     
     name = property(get_name, doc="Name of the potential")
     
