@@ -13,15 +13,18 @@ class ConstraintMixin:
         all dynamics supporting parallel MD.
         """
         # Store the arrays of the atoms, not the atoms, to prevent cyclic references.
-        self.atoms_arrays = atoms.arrays
-        assert self.indexname not in self.atoms_arrays
-        idx = self.index
-        self.asap_ready = True
-        if getattr(atoms, "parallel", False):
-            self.storable = False
-        self.index = idx
-        del self._index
-        assert self.indexname in self.atoms_arrays
+        if not self.asap_ready:
+            self.atoms_arrays = atoms.arrays
+            assert self.indexname not in self.atoms_arrays
+            idx = self.index
+            self.asap_ready = True
+            if getattr(atoms, "parallel", False):
+                self.storable = False
+            self.index = idx
+            del self._index
+            assert self.indexname in self.atoms_arrays
+        else:
+            assert self.indexname in self.atoms_arrays
         
     def set_index(self, idx):
         if self.asap_ready:
